﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace WCFService
{
	//
	// WCF 서비스 인스턴스 관리 테스트용 호스트
	//
	class WCFHost
	{
		static void Main(string[] args)
		{
			ServiceHost host1 = new ServiceHost(
				typeof(PerCallService), 
				new Uri("http://localhost:8083/wcf/instancing/percall"));
			// 코드에 의한 Throttling 설정
			//ServiceThrottlingBehavior behavior = new ServiceThrottlingBehavior();
			//behavior.MaxConcurrentCalls = 10;
			//behavior.MaxConcurrentInstances = 10;
			//behavior.MaxConcurrentSessions = 10;
			//host1.Description.Behaviors.Add(behavior);
			host1.Open();
			Console.WriteLine("Listening PerCall Service.......");

			ServiceHost host2 = new ServiceHost(
				typeof(PerSessionService),
				new Uri("http://localhost:8083/wcf/instancing/persession"));
			host2.Open();
			Console.WriteLine("Listening PerSession Service.......");

			ServiceHost host3 = new ServiceHost(
				new SingleService(5),
				new Uri("http://localhost:8083/wcf/instancing/single"));
			host3.Open();
			Console.WriteLine("Listening Single Service.......");

			ShowEndPoints(host1);
			ShowEndPoints(host2);
			ShowEndPoints(host3);

			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey();

			host1.Close();
			host2.Close();
			host3.Close();
		}

		static void ShowEndPoints(ServiceHost host)
		{
			Console.WriteLine("Information of Hosted Services....");

			int i = 1;
			foreach (System.ServiceModel.Description.ServiceEndpoint ep in host.Description.Endpoints) {
				Console.WriteLine("Endpoint #{0}", i++);
				Console.WriteLine("  Contract: {0}", ep.Contract.ContractType.FullName);
				Console.WriteLine("  Address: {0}", ep.Address.Uri.ToString());
				Console.WriteLine("  Binding: {0}", ep.Binding.Name);
				if (ep.Binding is WSHttpBinding) {
					WSHttpBinding binding = (WSHttpBinding)ep.Binding;
					Console.WriteLine("    Session: {0}", binding.ReliableSession.Enabled);
					Console.WriteLine("    Timeout: {0}", binding.ReliableSession.InactivityTimeout);
				}
				ServiceThrottlingBehavior behavior = host.Description.Behaviors.Find<ServiceThrottlingBehavior>();
				if (behavior != null) {
					Console.WriteLine("  Throttling:");
					Console.WriteLine("    Max Concurrent Sessions: {0}", behavior.MaxConcurrentSessions);
					Console.WriteLine("    Max Concurrent Calls: {0}", behavior.MaxConcurrentCalls);
					Console.WriteLine("    Max Concurrent Instances: {0}", behavior.MaxConcurrentInstances);
				}
			}
		}
	}
}
